import subprocess as sp


class IsddcCommand:
    exitcode = 0

    def __init__(self, cmd: str, *, timeout=None):
        self.cmd = cmd
        try:
            self.raw_output: str = sp.check_output(
                cmd,
                timeout=timeout,
                shell=True,
                text=True,
                stderr=sp.STDOUT)
        except sp.CalledProcessError as e:
            self.raw_output: str = e.output
            self.exitcode: int = e.returncode

    @property
    def raise_if_error(self):
        if self.exitcode != 0:
            raise CommandError(f'({self.cmd}) {self.output}')

    @property
    def status(self) -> bool:
        return False if self.exitcode else True

    @property
    def output(self) -> str:
        if self.raw_output[-1] == '\n':
            return self.raw_output[:-1]
        return self.raw_output

    @property
    def status_output(self) -> tuple:
        return self.status, self.output

    @property
    def code_output(self) -> tuple:
        return self.exitcode, self.output

    @property
    def output_else_exc(self) -> str:
        if self.exitcode == 0:
            return self.output
        raise CommandError(f'({self.cmd}) {self.output}')

    def is_contain_string(self, string: str) -> bool:
        if string in self.output:
            return True
        return False

    def output_if_contain_string_else_exc(self, string: str) -> str:
        if string in self.raw_output:
            return self.output
        raise CommandError(f'({self.cmd}): "{self.output}"')

    def table_output_to_dict(self, *, split: str = None) -> list:
        return self.table2dict(self.output_else_exc, split=split)

    @classmethod
    def table2dict(cls, table: str, *, split: str = None) -> list:
        """Used to turn the command output result with a title
        into a dictionary, such as `kubectl get pod`."""
        result = [[value.strip() for value in line.split(split)]
                  for line in table.splitlines()]
        keys = [key.lower() for key in result[0]]
        return [dict(zip(keys, values)) for values in result[1:]]


class CommandError(Exception):
    pass
